@@ -1,7 +1,5 @@
 pub mod buffer;
-pub mod cursor;
 pub mod edit_tree_viewer;
-pub mod emojis;
 pub mod prompt;
 pub mod splash;
 pub mod theme;
