@@ -11,12 +11,11 @@ use zi::ComponentLink;
 
 use super::{ContextHandle, Editor};
 use crate::{
-    components::cursor::Cursor,
+    edit::{strip_trailing_whitespace, Cursor, EditTree, TAB_WIDTH},
     error::Result,
     mode::{self, Mode, PLAIN_TEXT_MODE},
     syntax::parse::{OpaqueDiff, ParseTree, ParserPool, ParserStatus},
-    undo::EditTree,
-    utils::{strip_trailing_whitespace, TAB_WIDTH},
+    versioned::{Versioned, WeakHandle},
 };
 
 #[derive(Clone, Copy, Debug, PartialEq, Eq, Hash)]
@@ -142,74 +141,6 @@ impl Buffers {
     }
 }
 
-pub struct MainHandle<T> {
-    value: Rc<T>,
-    generation: usize,
-}
-
-impl<T> MainHandle<T> {
-    pub fn new(value: T) -> Self {
-        Self {
-            value: Rc::new(value),
-            generation: 0,
-        }
-    }
-
-    pub fn weak(&self) -> WeakHandle<T> {
-        WeakHandle {
-            value: Rc::downgrade(&self.value),
-            generation: self.generation,
-        }
-    }
-}
-
-impl<T> std::ops::Deref for MainHandle<T> {
-    type Target = T;
-
-    fn deref(&self) -> &T {
-        &self.value
-    }
-}
-
-impl<T: Clone> std::ops::DerefMut for MainHandle<T> {
-    fn deref_mut(&mut self) -> &mut T {
-        assert_eq!(Rc::strong_count(&self.value), 1);
-        self.generation += 1;
-        Rc::make_mut(&mut self.value)
-    }
-}
-
-#[derive(Clone)]
-pub struct WeakHandle<T> {
-    value: std::rc::Weak<T>,
-    generation: usize,
-}
-
-impl<T> WeakHandle<T> {
-    pub fn reader(&self) -> ReadHandle<T> {
-        ReadHandle(
-            self.value
-                .upgrade()
-                .expect("Tried deref-ing an invalid weak handle"),
-        )
-    }
-
-    pub fn generation(&self) -> usize {
-        self.generation
-    }
-}
-
-#[derive(Clone)]
-pub struct ReadHandle<T>(Rc<T>);
-
-impl<T> std::ops::Deref for ReadHandle<T> {
-    type Target = T;
-
-    fn deref(&self) -> &T {
-        &self.0
-    }
-}
-
 #[derive(Clone, Copy, Debug, PartialEq)]
 pub enum ModifiedStatus {
     Changed,
@@ -222,7 +153,7 @@ pub struct Buffer {
     id: BufferId,
     mode: &'static Mode,
     repo: Option<RepositoryRc>,
-    content: MainHandle<EditTree>,
+    content: Versioned<EditTree>,
     file_path: Option<PathBuf>,
     modified_status: ModifiedStatus,
     cursors: Vec<Cursor>,
@@ -250,14 +181,8 @@ impl Buffer {
                 || text.clone(),
                 move |status| {
                     link.send(
-                        BuffersMessage::new(
-                            id,
-                            BufferMessage::ParseSyntax {
-                                generation: 0,
-                                status,
-                            },
-                        )
-                        .into(),
+                        BuffersMessage::new(id, BufferMessage::ParseSyntax { version: 0, status })
+                            .into(),
                     )
                 },
             );
@@ -268,7 +193,7 @@ impl Buffer {
             id,
             mode,
             repo,
-            content: MainHandle::new(EditTree::new(text)),
+            content: Versioned::new(EditTree::new(text)),
             file_path,
             modified_status: ModifiedStatus::Unchanged,
             cursors: vec![Cursor::new()],
@@ -357,11 +282,11 @@ impl Buffer {
                 // self.properties.logger.info(error.to_string())
                 log::error!("{}", error);
             }
-            BufferMessage::ParseSyntax { generation, status } => {
+            BufferMessage::ParseSyntax { version, status } => {
                 log::info!("done!");
                 let parsed = status.unwrap();
                 if let Some(parser) = self.parser.as_mut() {
-                    parser.handle_parse_syntax_done(generation, parsed);
+                    parser.handle_parse_syntax_done(version, parsed);
                 }
             }
             BufferMessage::CursorMessage { cursor_id, message } => {
@@ -402,12 +327,14 @@ impl Buffer {
         let diff = {
             match message {
                 CursorMessage::DeleteForward => {
-                    let operation = self.cursors[cursor_id.0].delete(&mut self.content);
-                    // self.clipboard = Some(operation.deleted);
-                    operation.diff
+                    self.cursors[cursor_id.0]
+                        .delete_forward(&mut self.content)
+                        .diff
                 }
                 CursorMessage::DeleteBackward => {
-                    self.cursors[cursor_id.0].backspace(&mut self.content).diff
+                    self.cursors[cursor_id.0]
+                        .delete_backward(&mut self.content)
+                        .diff
                 }
                 CursorMessage::DeleteLine => self.delete_line(cursor_id),
                 CursorMessage::Yank => self.paste_from_clipboard(cursor_id),
@@ -462,8 +389,9 @@ impl Buffer {
     }
 
     fn delete_line(&mut self, cursor_id: CursorId) -> OpaqueDiff {
-        let operation = self.cursors[cursor_id.0].delete_line(&mut self.content);
-        operation.diff
+        self.cursors[cursor_id.0]
+            .delete_line(&mut self.content)
+            .diff
     }
 
     fn copy_selection_to_clipboard(&mut self, cursor_id: CursorId) -> OpaqueDiff {
@@ -526,15 +454,12 @@ impl Buffer {
             let staged_text = self.content.staged().clone();
             let buffer_id = self.id;
             let link = self.context.link.clone();
-            let generation = self.content.generation;
+            let version = self.content.version();
             parser.edit(diff);
             parser.spawn(task_pool, staged_text, fresh, move |status| {
                 link.send(
-                    BuffersMessage::new(
-                        buffer_id,
-                        BufferMessage::ParseSyntax { generation, status },
-                    )
-                    .into(),
+                    BuffersMessage::new(buffer_id, BufferMessage::ParseSyntax { version, status })
+                        .into(),
                 )
             });
         }
@@ -753,7 +678,7 @@ pub enum BufferMessage {
     SaveBufferStart,
     SaveBufferEnd(io::Result<Rope>),
     ParseSyntax {
-        generation: usize,
+        version: usize,
         status: Result<ParserStatus>,
     },
     PreviousChildRevision,
